@@ -66,6 +66,9 @@ class Move(metaclass=PoolMeta):
             move_qty = Uom.compute_qty(self.product.default_uom, move_qty,
                     move.uom, round=False)
             move.fifo_quantity = (move.fifo_quantity or 0.0) + move_qty
+            # Due to float, the fifo quantity result can exceed the quantity.
+            assert move.quantity >= move.fifo_quantity - move.uom.rounding
+            move.fifo_quantity = min(move.fifo_quantity, move.quantity)
             to_save.append(move)
         if to_save:
             # TODO save in do method when product change
@@ -126,3 +129,12 @@ class Move(metaclass=PoolMeta):
         if fifo_moves:
             cls.raise_user_error('del_move_fifo', (fifo_moves[0].rec_name,))
         super(Move, cls).delete(moves)
+
+    @classmethod
+    def copy(cls, moves, default=None):
+        if default is None:
+            default = {}
+        else:
+            default = default.copy()
+        default.setdefault('fifo_quantity', cls.default_fifo_quantity())
+        return super().copy(moves, default=default)
